/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.controller;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import java.util.Map;

import org.oransc.rappmanager.exception.ServiceException;
import org.oransc.rappmanager.service.ChartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController("chartController")
@RequestMapping("rms")
@Api(tags = {"chart"})
public class ChartController {

    @Autowired
    private ChartService chartService;

    private static Gson gson = new GsonBuilder().create();

    @GetMapping(path = "/charts", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Return all Charts")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "Chart List")})
    public ResponseEntity<Map<String, Chart[]>> getCharts() throws ServiceException {
        return new ResponseEntity<>(chartService.getCharts(), HttpStatus.OK);
    }

    @GetMapping(path = "/charts/{name}", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Return a chart")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "Chart")})
    public ResponseEntity<Chart[]> getChart(@PathVariable("name") String name) throws ServiceException {
        return new ResponseEntity<>(chartService.getChart(name), HttpStatus.OK);
    }

    @PostMapping(
        path = "/charts",
        consumes = MediaType.MULTIPART_FORM_DATA_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Onboard the Chart")
    @ApiResponses(value = {@ApiResponse(code = 201, message = "Chart Onboarded")})
    public ResponseEntity<String> onboardChart(@RequestParam("file") MultipartFile file,
        @RequestParam("chart") String chartJson) throws ServiceException {
        // TODO the json is a string here, but it should be Chart
        Chart chart = gson.fromJson(chartJson, Chart.class);

        return chartService.saveChart(chart, file).block();
    }

    @DeleteMapping(
        path = "/charts/{id}",
        consumes = MediaType.APPLICATION_JSON_VALUE,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "Delete the Chart")
    @ApiResponses(value = {@ApiResponse(code = 201, message = "Chart Deleted")})
    public ResponseEntity<Object> deleteChart(@PathVariable("name") String name,
        @PathVariable("version") String version) throws ServiceException {
        chartService.deleteChart(name, version);
        return new ResponseEntity<>(HttpStatus.CREATED);
    }

}
