/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.helm;

import java.util.Arrays;
import java.util.List;

import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class HelmUtil {

    private final String KUBE_API = "--kube-apiserver";
    private final String KUBE_TOKEN = "--kube-token";
    private final String KUBE_CERT = "--kube-ca-file";

    // Responsible for adding default chart registry to helm repo
    // Responsible for identifying the k8's config information required for helm
    @Autowired
    private ApplicationConfig appConfig;

    public ProcessBuilder prepareHelmCommand(String name, String version, String chartName, String releaseNamespace,
        String operation) {
        List<String> helmArguments = null;
        ProcessBuilder processBuilder = new ProcessBuilder();
        if ("install".equalsIgnoreCase(operation)) {
            helmArguments = Arrays.asList("helm", "install", name, chartName, "--version", version, "--namespace",
                releaseNamespace);
        } else if ("uninstall".equalsIgnoreCase(operation)) {
            helmArguments = Arrays.asList("helm", "uninstall", name, "--namespace", releaseNamespace);
        }
        addKubeApiInfo(helmArguments);
        return processBuilder.command(helmArguments);
    }

    private void addKubeApiInfo(List<String> helmArguments) {
        List<String> kubeApiInfo = Arrays.asList(KUBE_API, "", KUBE_TOKEN, "", KUBE_CERT, "");
        helmArguments.addAll(kubeApiInfo);
    }
}
