/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.service;

import org.oransc.rappmanager.exception.ServiceException;
import org.oransc.rappmanager.helm.HelmClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class AppService {

    @Autowired
    private AppStore appStore;

    @Autowired
    private HelmClient helmClient;

    public AppList getAllApps() throws ServiceException {
        // Call Kubernetes api to get the apps- pod, deplyment, statefulset, job?
        // Get the apps list this ms managing.
        return AppList.builder().apps(appStore.getAllApp()).build();
    }

    public App getApp(String name) {
        return appStore.getApp(name);
    }

    public App getAppStatus(String name) throws ServiceException {
        // Check the app status in the kubernetes cluster
        return appStore.getApp(name);
    }

    public void installApp(App app) throws ServiceException {
        // call the kubernetes api to create objects (helm call)
        // Call the kubernetes api to create namespace
        // Validate the override file
        if (helmClient.installApp(app)) {
            appStore.addApp(app);
        }
    }

    public void uninstallApp(String name, String releaseNamespace) throws ServiceException {
        // call the kubernetes api to delete objects (helm call)
        if (helmClient.uninstallApp(name, releaseNamespace)) {
            appStore.deleteApp(name);
        }
    }
}
