/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.oransc.rappmanager.exception.ServiceException;
import org.springframework.stereotype.Component;

@Component
public class AppStore {

    /**
     * The appStore map contains app name as key & App as value
     */
    private Map<String, App> appStore = new HashMap<>();

    public synchronized void addApp(App app) throws ServiceException {

        if (!appStore.containsKey(app.getName())) {
            appStore.put(app.getName(), app);
        } else {
            throw new ServiceException("App Already Exist");
        }
    }

    public synchronized App getApp(String name) {
        return appStore.get(name);
    }

    public synchronized List<App> getAllApp() {
        return new ArrayList<App>(appStore.values());
    }

    public synchronized void deleteApp(String name) {
        appStore.remove(name);
    }
}
