
package org.oransc.rappmanager.controller;

import static org.hamcrest.CoreMatchers.is;
import static org.mockito.BDDMockito.given;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import java.util.Arrays;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.oransc.rappmanager.Application;
import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.oransc.rappmanager.service.App;
import org.oransc.rappmanager.service.AppList;
import org.oransc.rappmanager.service.AppService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.MediaType;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT, classes = Application.class)
@AutoConfigureMockMvc
@EnableAutoConfiguration
@TestPropertySource(
    properties = { //
        "server.ssl.key-store=./config/keystore.jks", //
        "app.webclient.trust-store=./config/truststore.jks"})
public class AppControllerTest {

    @LocalServerPort
    private int port;

    @Autowired
    ApplicationConfig applicationConfig;

    @Autowired
    private MockMvc mvc;

    @MockBean
    private AppService appService;

    private AppList apps;

    @Test
    public void testGetAllApps() throws Exception {
        apps = getAllApps();
        given(appService.getAllApps()).willReturn(apps);
        mvc.perform(get("/rms/apps").contentType(MediaType.APPLICATION_JSON)).andExpect(status().isOk())
            .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
            .andExpect(jsonPath("$.apps.[0].name", is("helloworldApp")));
    }

    private AppList getAllApps() {
        return AppList.builder()
            .apps(Arrays.asList(App.builder().name("helloworldApp").releaseNamespace("helmrelease").build())).build();
    }
}
