
package org.oransc.rappmanager.controller;

import static org.assertj.core.api.Assertions.assertThat;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.oransc.rappmanager.Application;
import org.oransc.rappmanager.client.AsyncRestClient;
import org.oransc.rappmanager.client.AsyncRestClientFactory;
import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.oransc.rappmanager.configuration.ImmutableWebClientConfig;
import org.oransc.rappmanager.configuration.WebClientConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.MultipartBodyBuilder;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.BodyInserters.MultipartInserter;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT, classes = Application.class)
@AutoConfigureMockMvc
@EnableAutoConfiguration
@TestPropertySource(
    properties = { //
        "server.ssl.key-store=./config/keystore.jks", //
        "app.webclient.trust-store=./config/truststore.jks"})
public class ChartControllerTest {

    @LocalServerPort
    private int port;

    @Autowired
    private MockMvc mvc;

    // @MockBean
    // private ChartService chartService;

    @Autowired
    ApplicationConfig applicationConfig;

    private Map<String, Chart[]> charts = new HashMap<>();

    // @Test
    // public void testGetCharts() throws Exception {
    // chartList();
    // given(chartService.getCharts()).willReturn(charts);
    // mvc.perform(get("/rms/charts").contentType(MediaType.APPLICATION_JSON)).andExpect(status().isOk())
    // .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON))
    // .andExpect(jsonPath("$.helloworldApp.[0].name", is("helloworldApp")));
    // }

    @Test
    public void testPutChart() {
        String url = "/rms/charts";
        String fileName = "config/application.yaml"; // for now
        AsyncRestClient client = restClient();
        Chart chart = new Chart();
        chart.setName("TEST");
        MultipartInserter body = createPostChartBody(new File(fileName), chart);
        ResponseEntity<String> e = client.postForEntity(url, body, MediaType.MULTIPART_FORM_DATA).block();
        assertThat(e.getStatusCode()).isEqualTo(HttpStatus.OK);
    }

    private MultipartInserter createPostChartBody(File file, Chart chart) {
        MultipartBodyBuilder bodyBuilder = new MultipartBodyBuilder();
        bodyBuilder.part("file", new FileSystemResource(file));
        bodyBuilder.part("chart", chart);
        return BodyInserters.fromMultipartData(bodyBuilder.build());
    }

    private void chartList() {
        Chart chart = new Chart();
        chart.setName("helloworldApp");
        Chart[] chartArray = new Chart[1];
        chartArray[0] = chart;
        charts.put("helloworldApp", chartArray);
    }

    private AsyncRestClient restClient(String baseUrl, boolean useTrustValidation) {
        WebClientConfig config = this.applicationConfig.getWebClientConfig();
        config = ImmutableWebClientConfig.builder() //
            .keyStoreType(config.keyStoreType()) //
            .keyStorePassword(config.keyStorePassword()) //
            .keyStore(config.keyStore()) //
            .keyPassword(config.keyPassword()) //
            .isTrustStoreUsed(useTrustValidation) //
            .trustStore(config.trustStore()) //
            .trustStorePassword(config.trustStorePassword()) //
            .httpProxyConfig(config.httpProxyConfig()) //
            .build();

        AsyncRestClientFactory f = new AsyncRestClientFactory(config);
        return f.createRestClientNoHttpProxy(baseUrl);
    }

    private AsyncRestClient restClient(boolean useTrustValidation) {
        String baseUrl = "https://localhost:" + port;
        return restClient(baseUrl, useTrustValidation);
    }

    private AsyncRestClient restClient() {
        return restClient(false);
    }
}
